@@ -1,12 +1,18 @@
 from collections import defaultdict
-from typing import Union, Optional, List
+from typing import Union, Optional, List, Dict
 
 from alpaca.common.constants import DATA_V2_MAX_LIMIT
 from alpaca.common.types import RawData
 from alpaca.common.enums import BaseURL
 from alpaca.common.rest import RESTClient
-from alpaca.common.types import HTTPResult, Credentials
-from alpaca.data.models import BarSet, QuoteSet, TradeSet, SnapshotSet
+from alpaca.common.types import Credentials
+from alpaca.data import Snapshot, Bar
+from alpaca.data.historical.utils import (
+    parse_obj_as_symbol_dict,
+    format_latest_data_response,
+    format_dataset_response,
+)
+from alpaca.data.models import BarSet, QuoteSet, TradeSet, Orderbook, Trade, Quote
 from alpaca.data.historical.stock import DataExtensionType
 from alpaca.data.requests import (
     CryptoBarsRequest,
@@ -15,6 +21,8 @@ from alpaca.data.requests import (
     CryptoLatestTradeRequest,
     CryptoLatestQuoteRequest,
     CryptoSnapshotRequest,
+    CryptoLatestOrderbookRequest,
+    CryptoLatestBarRequest,
 )
 
 
@@ -47,7 +55,7 @@ class CryptoHistoricalDataClient(RESTClient):
             secret_key (Optional[str], optional): Alpaca API secret key. Defaults to None.
             oauth_token (Optional[str]): The oauth token if authenticating via OAuth. Defaults to None.
             raw_data (bool, optional): If true, API responses will not be wrapped and raw responses will be returned from
-              methods. Defaults to False.
+              methods. Defaults to False. This has not been implemented yet.
             url_override (Optional[str], optional): If specified allows you to override the base url the client points
               to for proxy/testing.
         """
@@ -55,7 +63,7 @@ class CryptoHistoricalDataClient(RESTClient):
             api_key=api_key,
             secret_key=secret_key,
             oauth_token=oauth_token,
-            api_version="v2",
+            api_version="v1beta2",
             base_url=url_override if url_override is not None else BaseURL.DATA,
             sandbox=False,
             raw_data=raw_data,
@@ -83,7 +91,7 @@ class CryptoHistoricalDataClient(RESTClient):
             **params,
         )
 
-        return self.response_wrapper(model=BarSet, raw_data=raw_bars)
+        return BarSet(raw_bars)
 
     def get_crypto_quotes(
         self, request_params: CryptoQuotesRequest
@@ -107,12 +115,13 @@ class CryptoHistoricalDataClient(RESTClient):
             **params,
         )
 
-        return self.response_wrapper(model=QuoteSet, raw_data=raw_quotes)
+        return QuoteSet(raw_quotes)
 
     def get_crypto_trades(
         self, request_params: CryptoTradesRequest
     ) -> Union[TradeSet, RawData]:
-        """Returns the price and sales history over a given time period for a cryptocurrency or list of cryptocurrencies.
+        """Returns the price and sales history over a given time period for a cryptocurrency
+        or list of cryptocurrencies.
 
         Args:
             request_params (CryptoTradesRequest): The parameters for the request.
@@ -131,18 +140,18 @@ class CryptoHistoricalDataClient(RESTClient):
             **params,
         )
 
-        return self.response_wrapper(model=TradeSet, raw_data=raw_trades)
+        return TradeSet(raw_trades)
 
     def get_crypto_latest_trade(
         self, request_params: CryptoLatestTradeRequest
-    ) -> Union[TradeSet, RawData]:
-        """Returns the latest trade for a coin for a specific exchange
+    ) -> Union[Dict[str, Trade], RawData]:
+        """Returns the latest trade for a coin.
 
         Args:
             request_params (CryptoLatestTradeRequest): The parameters for the request.
 
         Returns:
-            Union[Trade, RawData]: The latest trade in raw or wrapped format
+            Union[Dict[str, Trade], RawData]: The latest trade in raw or wrapped format
         """
 
         params = request_params.to_request_fields()
@@ -155,18 +164,18 @@ class CryptoHistoricalDataClient(RESTClient):
             **params,
         )
 
-        return self.response_wrapper(model=TradeSet, raw_data=raw_trades)
+        return parse_obj_as_symbol_dict(Trade, raw_trades)
 
     def get_crypto_latest_quote(
         self, request_params: CryptoLatestQuoteRequest
-    ) -> Union[QuoteSet, RawData]:
-        """Returns the latest quote for a coin for a specific exchange
+    ) -> Union[Dict[str, Quote], RawData]:
+        """Returns the latest quote for a coin.
 
         Args:
             request_params (CryptoLatestQuoteRequest): The parameters for the request.
 
         Returns:
-            Union[Quote, RawData]: The latest quote in raw or wrapped format
+            Union[Dict[str, Quote], RawData]: The latest quote in raw or wrapped format
         """
 
         params = request_params.to_request_fields()
@@ -179,16 +188,65 @@ class CryptoHistoricalDataClient(RESTClient):
             **params,
         )
 
-        return self.response_wrapper(model=QuoteSet, raw_data=raw_quotes)
+        return parse_obj_as_symbol_dict(Quote, raw_quotes)
+
+    def get_crypto_latest_bar(
+        self, request_params: CryptoLatestBarRequest
+    ) -> Union[Dict[str, Bar], RawData]:
+        """Returns the latest minute bar for a coin.
+
+        Args:
+            request_params (CryptoLatestBarRequest): The parameters for the request.
+
+        Returns:
+            Union[Dict[str, Bar], RawData]: The latest bar in raw or wrapped format
+        """
+
+        params = request_params.to_request_fields()
+
+        raw_bars = self._data_get(
+            endpoint_asset_class="crypto",
+            endpoint_data_type="bars",
+            api_version="v1beta2",
+            extension=DataExtensionType.LATEST,
+            **params,
+        )
+
+        return parse_obj_as_symbol_dict(Bar, raw_bars)
+
+    def get_crypto_latest_orderbook(
+        self, request_params: CryptoLatestOrderbookRequest
+    ) -> Union[Dict[str, Orderbook], RawData]:
+        """
+        Returns the latest orderbook state for the queried crypto symbols.
+
+        Args:
+            request_params (CryptoOrderbookRequest): The parameters for the orderbook request.
+
+        Returns:
+            Union[Dict[str, Orderbook], RawData]: The orderbook data either in raw or wrapped form.
+        """
+
+        params = request_params.to_request_fields()
+
+        raw_orderbooks = self._data_get(
+            endpoint_asset_class="crypto",
+            endpoint_data_type="orderbooks",
+            api_version="v1beta2",
+            extension=DataExtensionType.LATEST,
+            **params,
+        )
+
+        return parse_obj_as_symbol_dict(Orderbook, raw_orderbooks)
 
     def get_crypto_snapshot(
         self, request_params: CryptoSnapshotRequest
-    ) -> Union[SnapshotSet, RawData]:
+    ) -> Union[Snapshot, RawData]:
         """Returns snapshots of queried crypto symbols. Snapshots contain latest trade, latest quote, latest minute bar,
         latest daily bar and previous daily bar data for the queried symbols.
 
         Args:
-            request_params (CryptoSnapshotRequest): The parameters for the request.
+            request_params (CryptoSnapshotRequest): The parameters for the snapshot request.
 
         Returns:
             Union[SnapshotSet, RawData]: The snapshot data either in raw or wrapped form
@@ -203,11 +261,8 @@ class CryptoHistoricalDataClient(RESTClient):
             extension=DataExtensionType.SNAPSHOT,
             **params,
         )
-        print(raw_snapshots)
-        return self.response_wrapper(
-            model=SnapshotSet,
-            raw_data=raw_snapshots,
-        )
+
+        return parse_obj_as_symbol_dict(Snapshot, raw_snapshots)
 
     # TODO: Remove duplicate code
     def _data_get(
@@ -230,7 +285,8 @@ class CryptoHistoricalDataClient(RESTClient):
             endpoint_asset_class (str): The data API security type path. Defaults to 'stocks'.
             api_version (str): Data API version. Defaults to "v2".
             limit (Optional[int]): The maximum number of items to query. Defaults to None.
-            page_limit (Optional[int]): The maximum number of items returned per page - different from limit. Defaults to DATA_V2_MAX_LIMIT.
+            page_limit (Optional[int]): The maximum number of items returned per page - different from limit.
+                Defaults to DATA_V2_MAX_LIMIT.
 
         Returns:
             RawData: Raw Market data from API
@@ -262,6 +318,7 @@ class CryptoHistoricalDataClient(RESTClient):
         #       "symbol2": [ "data1", "data2", ... ],
         #                ....
         #    }
+        # using default dict to improve parsing,
         data_by_symbol = defaultdict(list)
 
         total_items = 0
@@ -284,10 +341,13 @@ class CryptoHistoricalDataClient(RESTClient):
             response = self.get(path=path, data=params, api_version=api_version)
 
             # TODO: Merge parsing if possible
-            if extension == DataExtensionType.SNAPSHOT:
-                self._parse_snapshot(response, data_by_symbol)
+            if (
+                extension == DataExtensionType.LATEST
+                or extension == DataExtensionType.SNAPSHOT
+            ):
+                format_latest_data_response(response, data_by_symbol)
             else:
-                self._parse_response(response, data_by_symbol)
+                format_dataset_response(response, data_by_symbol)
 
             # if we've sent a request with a limit, increment count
             if actual_limit:
@@ -298,71 +358,8 @@ class CryptoHistoricalDataClient(RESTClient):
             if page_token is None:
                 break
 
-        return data_by_symbol
-
-    @staticmethod
-    def _parse_response(response: HTTPResult, data_by_symbol: dict) -> RawData:
-
-        # data_by_symbol is in format of
-        #    {
-        #       "symbol1": [ "data1", "data2", ... ],
-        #       "symbol2": [ "data1", "data2", ... ],
-        #                ....
-        #    }
-
-        response_data = CryptoHistoricalDataClient.get_data_from_response(response)
-
-        # add elements to data_by_symbol
-        # for list data types just extend
-        # for non-list types, add as element of a list.
-        # list comprehension used for speed
-        [
-            data_by_symbol[symbol].extend(data)
-            if isinstance(data, list)
-            else data_by_symbol[symbol].append(data)
-            for symbol, data in response_data.items()
-        ]
-
-        return data_by_symbol
-
-    @staticmethod
-    def get_data_from_response(response: HTTPResult) -> RawData:
-
-        data_keys = {
-            "trade",
-            "trades",
-            "quote",
-            "quotes",
-            "bar",
-            "bars",
-            "snapshot",
-            "snapshots",
-        }
-
-        selected_key = data_keys.intersection(response)
-
-        if selected_key is None or len(selected_key) < 1:
-            raise ValueError("The data in response does not match any known keys.")
-
-        # assume selected_key only contains 1 value
-        selected_key = selected_key.pop()
-
-        # formatting a single symbol response so that this method
-        # always returns a symbol keyed data dictionary
-        if "symbol" in response:
-            return {response["symbol"]: response[selected_key]}
-
-        return response[selected_key]
-
-    @staticmethod
-    def _parse_snapshot(response: HTTPResult, data_by_symbol: dict):
-        # TODO: Improve snapshot parsing
-        response_data = CryptoHistoricalDataClient.get_data_from_response(response)
-
-        for symbol, data in response_data.items():
-            data_by_symbol[symbol] = data
-
-        return data_by_symbol
+        # users receive Type dict
+        return dict(data_by_symbol)
 
     @staticmethod
     def _validate_credentials(
